@@ -10,7 +10,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 import horizon
 
@@ -19,6 +19,9 @@ class GroupTypes(horizon.Panel):
     name = _("Group Types")
     slug = 'group_types'
     permissions = (
-        ('openstack.services.volume', 'openstack.services.volumev3'),
+        ('openstack.services.block-storage',
+         'openstack.services.block-store',
+         'openstack.services.volume',
+         'openstack.services.volumev3'),
     )
     policy_rules = (("volume", "group:group_types_manage"),)
