@@ -56,16 +56,13 @@ class NetworkPortTests(test.TestCase):
 
     @test.create_mocks({api.neutron: ('network_get',
                                       'port_get',
-                                      'is_extension_supported',
-                                      'security_group_list')})
+                                      'security_group_list',
+                                      'is_extension_supported')})
     def _test_port_detail(self, mac_learning=False):
         # Use a port associated with security group
-        port = [p for p in self.ports.list() if p.security_groups][0]
-        sgs = [sg for sg in self.security_groups.list()
-               if sg.id in port.security_groups]
+        port = [p for p in self.ports.list() if p.security_group_ids][0]
         network_id = self.networks.first().id
         self.mock_port_get.return_value = port
-        self.mock_security_group_list.return_value = sgs
         self._stub_is_extension_supported({'mac-learning': mac_learning,
                                            'allowed-address-pairs': False})
         self.mock_network_get.return_value = self.networks.first()
@@ -77,9 +74,6 @@ class NetworkPortTests(test.TestCase):
 
         self.mock_port_get.assert_called_once_with(test.IsHttpRequest(),
                                                    port.id)
-        self.mock_security_group_list.assert_called_once_with(
-            test.IsHttpRequest(),
-            id=tuple(sg.id for sg in port.security_groups))
         self._check_is_extension_supported({'mac-learning': 2,
                                             'allowed-address-pairs': 2})
         self.mock_network_get.assert_called_once_with(test.IsHttpRequest(),
@@ -168,7 +162,7 @@ class NetworkPortTests(test.TestCase):
         form_data = {'network_id': port.network_id,
                      'port_id': port.id,
                      'name': port.name,
-                     'admin_state': port.admin_state_up}
+                     'admin_state': port.is_admin_state_up}
         if binding:
             form_data['binding__vnic_type'] = port.binding__vnic_type
         if mac_learning:
@@ -185,16 +179,18 @@ class NetworkPortTests(test.TestCase):
         self.assertRedirectsNoFollow(res, redir_url)
 
         self.assert_mock_multiple_calls_with_same_arguments(
-            self.mock_port_get, 2,
+            self.mock_port_get, 3,
             mock.call(test.IsHttpRequest(), port.id))
         self._check_is_extension_supported({'binding': 1,
                                             'mac-learning': 1,
                                             'port-security': 1})
         self.mock_security_group_list.assert_called_once_with(
             test.IsHttpRequest(), tenant_id=self.tenant.id)
+        if form_data.get('port_security_enabled') == port.port_security_enabled:
+            extension_kwargs.pop('port_security_enabled')
         self.mock_port_update.assert_called_once_with(
             test.IsHttpRequest(), port.id, name=port.name,
-            admin_state_up=port.admin_state_up,
+            admin_state_up=port.is_admin_state_up,
             **extension_kwargs)
 
     def test_port_update_post_exception(self):
@@ -227,7 +223,7 @@ class NetworkPortTests(test.TestCase):
         form_data = {'network_id': port.network_id,
                      'port_id': port.id,
                      'name': port.name,
-                     'admin_state': port.admin_state_up}
+                     'admin_state': port.is_admin_state_up}
         if binding:
             form_data['binding__vnic_type'] = port.binding__vnic_type
         if mac_learning:
@@ -244,7 +240,7 @@ class NetworkPortTests(test.TestCase):
         self.assertRedirectsNoFollow(res, redir_url)
 
         self.assert_mock_multiple_calls_with_same_arguments(
-            self.mock_port_get, 2,
+            self.mock_port_get, 3,
             mock.call(test.IsHttpRequest(), port.id))
         self._check_is_extension_supported({'binding': 1,
                                             'mac-learning': 1,
@@ -259,9 +255,11 @@ class NetworkPortTests(test.TestCase):
         if port_security:
             extension_kwargs['port_security_enabled'] = True
         extension_kwargs['security_groups'] = sg_ids
+        if form_data.get('port_security_enabled') == port.port_security_enabled:
+            extension_kwargs.pop('port_security_enabled')
         self.mock_port_update.assert_called_once_with(
             test.IsHttpRequest(), port.id, name=port.name,
-            admin_state_up=port.admin_state_up,
+            admin_state_up=port.is_admin_state_up,
             **extension_kwargs)
 
     @test.create_mocks({api.neutron: ('port_get',
@@ -293,6 +291,7 @@ class NetworkPortTests(test.TestCase):
     @override_settings(POLICY_CHECK_FUNCTION='openstack_auth.policy.check')
     @test.create_mocks({api.neutron: ('port_get',
                                       'network_get',
+                                      'security_group_list',
                                       'is_extension_supported')})
     def test_add_allowed_address_pair_button_shown_to_network_owner(self):
         port = self.ports.first()
@@ -410,6 +409,7 @@ class NetworkPortTests(test.TestCase):
     @override_settings(POLICY_CHECK_FUNCTION='openstack_auth.policy.check')
     @test.create_mocks({api.neutron: ('port_get',
                                       'network_get',
+                                      'security_group_list',
                                       'port_update',
                                       'is_extension_supported')})
     def test_delete_address_pair_button_shown_to_network_owner(self):
@@ -482,8 +482,8 @@ class NetworkPortTests(test.TestCase):
         self._stub_is_extension_supported({'mac-learning': False,
                                            'allowed-address-pairs': True})
 
-        pair_ip = pair['ip_address']
-        form_data = {'action': 'allowed_address_pairs__delete__%s' % pair_ip}
+        id = pair['ip_address'] + ":" + pair['mac_address'].replace(':', '-')
+        form_data = {'action': 'allowed_address_pairs__delete__%s' % id}
         url = reverse(detail_path, args=[pre_port.id])
 
         res = self.client.post(url, form_data)
@@ -543,6 +543,37 @@ class NetworkPortTests(test.TestCase):
         self.mock_security_group_list.assert_called_once_with(
             test.IsHttpRequest(), tenant_id='1')
 
+    @test.create_mocks({api.neutron: ('network_get',
+                                      'security_group_list',
+                                      'is_extension_supported')})
+    def test_port_create_on_network_from_different_tenant(self):
+        network = self.networks.list()[1]
+        tenant_id = self.request.user.tenant_id
+        # Ensure the network belongs to a different tenant
+        self.assertNotEqual(tenant_id, network.tenant_id)
+
+        self.mock_network_get.return_value = self.networks.first()
+        self._stub_is_extension_supported({'binding': False,
+                                           'mac-learning': False,
+                                           'port-security': True})
+        self.mock_security_group_list.return_value = \
+            self.security_groups.list()
+
+        url = reverse('horizon:project:networks:addport',
+                      args=[network.id])
+        res = self.client.get(url)
+
+        self.assertTemplateUsed(res, views.WorkflowView.template_name)
+        self.assert_mock_multiple_calls_with_same_arguments(
+            self.mock_network_get, 2,
+            mock.call(test.IsHttpRequest(), network.id))
+        self._check_is_extension_supported({'binding': 1,
+                                            'mac-learning': 1,
+                                            'port-security': 1})
+        # Check the new port belongs to a tenant of the login user
+        self.mock_security_group_list.assert_called_once_with(
+            test.IsHttpRequest(), tenant_id=tenant_id)
+
     def test_port_create_post(self):
         self._test_port_create_post()
 
@@ -571,7 +602,7 @@ class NetworkPortTests(test.TestCase):
         form_data = {'network_id': port.network_id,
                      'network_name': network.name,
                      'name': port.name,
-                     'admin_state': port.admin_state_up,
+                     'admin_state': port.is_admin_state_up,
                      'device_id': port.device_id,
                      'device_owner': port.device_owner,
                      'specify_ip': 'fixed_ip',
@@ -593,7 +624,7 @@ class NetworkPortTests(test.TestCase):
         self.assertRedirectsNoFollow(res, redir_url)
 
         self.assert_mock_multiple_calls_with_same_arguments(
-            self.mock_network_get, 2,
+            self.mock_network_get, 3,
             mock.call(test.IsHttpRequest(), network.id))
         self._check_is_extension_supported({'binding': 1,
                                             'mac-learning': 1,
@@ -612,7 +643,7 @@ class NetworkPortTests(test.TestCase):
             test.IsHttpRequest(),
             network_id=network.id,
             name=port.name,
-            admin_state_up=port.admin_state_up,
+            admin_state_up=port.is_admin_state_up,
             device_id=port.device_id,
             device_owner=port.device_owner,
             fixed_ips=fixed_ips,
@@ -637,7 +668,7 @@ class NetworkPortTests(test.TestCase):
         form_data = {'network_id': port.network_id,
                      'network_name': network.name,
                      'name': port.name,
-                     'admin_state': port.admin_state_up,
+                     'admin_state': port.is_admin_state_up,
                      'device_id': port.device_id,
                      'device_owner': port.device_owner,
                      'specify_ip': 'subnet_id',
@@ -671,7 +702,7 @@ class NetworkPortTests(test.TestCase):
         form_data = {'network_id': port.network_id,
                      'network_name': network.name,
                      'name': port.name,
-                     'admin_state': port.admin_state_up,
+                     'admin_state': port.is_admin_state_up,
                      'device_id': port.device_id,
                      'device_owner': port.device_owner,
                      'specify_ip': 'fixed_ip',
@@ -715,7 +746,7 @@ class NetworkPortTests(test.TestCase):
         form_data = {'network_id': port.network_id,
                      'network_name': network.name,
                      'name': port.name,
-                     'admin_state': port.admin_state_up,
+                     'admin_state': port.is_admin_state_up,
                      'mac_state': True,
                      'device_id': port.device_id,
                      'device_owner': port.device_owner,
@@ -738,7 +769,7 @@ class NetworkPortTests(test.TestCase):
         self.assertRedirectsNoFollow(res, redir_url)
 
         self.assert_mock_multiple_calls_with_same_arguments(
-            self.mock_network_get, 2,
+            self.mock_network_get, 3,
             mock.call(test.IsHttpRequest(), network.id))
         self._check_is_extension_supported({'binding': 1,
                                             'mac-learning': 1,
@@ -755,7 +786,7 @@ class NetworkPortTests(test.TestCase):
             test.IsHttpRequest(),
             network_id=network.id,
             name=port.name,
-            admin_state_up=port.admin_state_up,
+            admin_state_up=port.is_admin_state_up,
             device_id=port.device_id,
             device_owner=port.device_owner,
             fixed_ips=fixed_ips,
@@ -790,8 +821,10 @@ class NetworkPortTests(test.TestCase):
 
         self.mock_port_delete.assert_called_once_with(test.IsHttpRequest(),
                                                       port.id)
-        self.mock_port_list.assert_called_once_with(test.IsHttpRequest(),
-                                                    network_id=network_id)
+        self.mock_port_list.assert_called_once_with(
+            test.IsHttpRequest(),
+            network_id=network_id,
+            project_id=self.tenant.id)
         self.mock_is_extension_supported.assert_called_once_with(
             test.IsHttpRequest(), 'mac-learning')
 
@@ -820,7 +853,9 @@ class NetworkPortTests(test.TestCase):
 
         self.mock_port_delete.assert_called_once_with(test.IsHttpRequest(),
                                                       port.id)
-        self.mock_port_list.assert_called_once_with(test.IsHttpRequest(),
-                                                    network_id=network_id)
+        self.mock_port_list.assert_called_once_with(
+            test.IsHttpRequest(),
+            network_id=network_id,
+            project_id=self.tenant.id)
         self.mock_is_extension_supported.assert_called_once_with(
             test.IsHttpRequest(), 'mac-learning')
